/********************************************************************************
 *
 *   Program:  STM32F042 USB CDC Example      Version: 1.00
 *
 *   Author:  S. Swift					      Date: September 7, 2014
 *
 *   This test example application configures the STM32F042 as a CDC USB device
 *   using the CooCox V1.7.7 IDE and STMicroelectronics STM32F0x2_USB-FS-Device_Lib V1.0.0.
 *
 *   This software can be used without restriction as the basis for other applications
 *   and products.  The STM32F042 is a low cost ($2.50) ARM M0 processor running at 48MHz
 *   with 32K of flash, 6K SRAM, A/D, 2 SPI, 2 USARTs, I2C,  etc in 32 and 64
 *   pin QFP and QFN package types.  A good, low cost, processor.  We are building the F042
 *   into several new products using this code as a starting point.
 *
 *   The application configures the USB peripheral as a CDC device, USART1 to 38,400,N,8,1,
 *   TIM3 as 1msec time delay clock, TIM15 as a usec delay timer. A printf, scanf function is
 *   available for USB CDC and putchar,getchar for USART1.  USART1 is interrupt driven.
 *
 *   The RF24L01 code was ported but not configured or tested with SPI - TBD.
 *
 *   Unless required by applicable law or agreed to in writing, software distributed under
 *   the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 *   ANY KIND, either express or implied. See the License for the specific language governing
 *   permissions and limitations under the License. Use at your own risk.
 *
 *   Copyright 2014,  SOC Robotics, Inc.
 ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2013 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
 */
/* Includes ------------------------------------------------------------------*/
#include "stm32f0xx.h"
#include "stm32f0xx_conf.h"
#include "stm32072b_eval.h"

#include "usbd_cdc_core.h"
#include "usbd_usr.h"

#include "stdarg.h"
#include "string.h"
#include "math.h"
#include <stdio.h>
#include <stdlib.h>

USB_CORE_HANDLE  USB_Device_dev ;

/* Private function prototypes -----------------------------------------------*/
void LEDInit(void);
void EXTI0_Config(void);
void TIM_Config(void);
void USARTInit(unsigned long baudrate);

#define 	XON   		  0x11
#define 	XOFF  		  0x13
#define 	LF	  		  0x0A
#define 	CR	  		  0x0D
#define  	SPACE 	   	  0x20

#define LED1ON  GPIO_ResetBits(GPIOB,GPIO_Pin_0);
#define LED1OFF  GPIO_SetBits(GPIOB,GPIO_Pin_0);
#define LED2ON  GPIO_ResetBits(GPIOB,GPIO_Pin_1);
#define LED2OFF  GPIO_SetBits(GPIOB,GPIO_Pin_1);

#define LED1HIGH   GPIOB->BSRR = GPIO_Pin_0
#define LED1LOW	   GPIOB->BRR = GPIO_Pin_0
#define LED2HIGH   GPIOB->BSRR = GPIO_Pin_1
#define LED2LOW	   GPIOB->BRR = GPIO_Pin_1

// STM32F042 defines
#define PWMACTIVE  'a'
#define CMDACTIVE  'c'
#define INACTIVE 'i'
#define YES     'y'
#define NO      'n'
#define HIGH	'h'
#define LOW		'l'

unsigned int mseccountdown1,mseccountdown2,mseccountdown3;
char flip=0,grnflip=0,redflip=0;

#define     USB_RX_BUFFERSIZE 256

int usbrxheadptr=0, usbrxtailptr=0;
unsigned char usbrxbuffer[USB_RX_BUFFERSIZE];

#define     USART1_TX_BUFSIZE 80
#define     USART1_RX_BUFSIZE 80

int tx1headptr,tx1tailptr,rx1headptr,rx1tailptr;
unsigned char rx1buffer[USART1_RX_BUFSIZE],tx1buffer[USART1_TX_BUFSIZE];

TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
TIM_OCInitTypeDef  TIM_OCInitStructure;
TIM_ICInitTypeDef  TIM_ICInitStructure;

/***********************************************
 *
 *  Initialization Code
 *
 ***********************************************/
// Initialize and start USB CDC
void USB_Init(void)
{
	USBD_Init(&USB_Device_dev,
	            &USR_desc,
	            &USBD_CDC_cb,
	            &USR_cb);
}

//  Configures LED GPIO.
void LED_Init(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;

	/* Enable the GPIO_LED Clock */
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

	/* Configure the GPIO_LED pin */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1;
	//  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	//  GPIO_Init(GPIOA, &GPIO_InitStructure);
	GPIO_Init(GPIOB, &GPIO_InitStructure);
}

// Configure the TIM IRQ Handler.
void TIM3_Config(void)
{
	NVIC_InitTypeDef NVIC_InitStructure;

	/* TIM3 clock enable */
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);

	/* Enable the TIM3 global Interrupt */
	NVIC_InitStructure.NVIC_IRQChannel = TIM3_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
}

// Setup Timer 3 as a 1msec time delay interrupt
void TIM3_Init(void)
{
	uint16_t CCR3_Val,PrescalerValue;

	/* TIM Configuration */
	TIM3_Config();

	/* -----------------------------------------------------------------------
	TIM3 Configuration: Output Compare Timing Mode:

	In this example TIM3 input clock (TIM3CLK) is set to APB1 clock (PCLK1),
	  => TIM3CLK = PCLK1 = SystemCoreClock = 48 MHz

	To get TIM3 counter clock at 6 MHz, the prescaler is computed as follows:
	   Prescaler = (TIM3CLK / TIM3 counter clock) - 1
	   Prescaler = (PCLK1 /6 MHz) - 1

	CC3 update rate = TIM3 counter clock / CCR3_Val = 439.4 Hz
	==> Toggling frequency = 219.7 Hz

	----------------------------------------------------------------------- */

	/* Compute the prescaler value */
	PrescalerValue = (uint16_t) (SystemCoreClock  / 6000000) - 1;
	//PrescalerValue = 0;

	CCR3_Val = 1000;
	mseccountdown1 = mseccountdown2 = mseccountdown3 = 0;

	/* Time base configuration */
	TIM_TimeBaseStructure.TIM_Period = 65535;
	TIM_TimeBaseStructure.TIM_Prescaler = 0;
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

	TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);

	/* Prescaler configuration */
	TIM_PrescalerConfig(TIM3, PrescalerValue, TIM_PSCReloadMode_Immediate);

	/* Output Compare Timing Mode configuration: Channel1 */
	//TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Timing;
	//TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;

	/* Output Compare Timing Mode configuration: Channel3 */
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	TIM_OCInitStructure.TIM_Pulse = CCR3_Val;

	TIM_OC3Init(TIM3, &TIM_OCInitStructure);

	TIM_OC3PreloadConfig(TIM3, TIM_OCPreload_Disable);

	/* TIM Interrupts enable */
	TIM_ITConfig(TIM3, TIM_IT_CC3, ENABLE);

	/* TIM3 enable counter */
	TIM_Cmd(TIM3, ENABLE);
}

// Set up timer 15 as usecond delay timer
void TIM15_Init(void)
{
	uint16_t CCR3_Val,PrescalerValue;

	/* TIM3 clock enable */
	RCC_APB1PeriphClockCmd(RCC_APB2Periph_TIM15, ENABLE);

	/* Compute the prescaler value for 1usecond rate */
	PrescalerValue = (uint16_t) (SystemCoreClock  / 1000000) - 1;
	//PrescalerValue = 0;

	CCR3_Val = 0;

	/* Time base configuration */
	TIM_TimeBaseStructure.TIM_Period = 65535;
	TIM_TimeBaseStructure.TIM_Prescaler = 0;
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

	TIM_TimeBaseInit(TIM15, &TIM_TimeBaseStructure);

	/* Prescaler configuration */
	TIM_PrescalerConfig(TIM15, PrescalerValue, TIM_PSCReloadMode_Immediate);

	/* Output Compare Timing Mode configuration: Channel3 */
	//TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	//TIM_OCInitStructure.TIM_Pulse = CCR3_Val;

	//TIM_OC3Init(TIM15, &TIM_OCInitStructure);

	//TIM_OC1PreloadConfig(TIM15, TIM_OCPreload_Disable);

	/* TIM Interrupts enable */
	//TIM_ITConfig(TIM15, TIM_IT_CC1, ENABLE);

	/* TIM3 enable counter */
	//TIM_Cmd(TIM15, ENABLE);
}

void TIM15_Start(void)
{
	TIM_SetCounter(TIM15, 0);
	TIM_Cmd(TIM15, ENABLE);
}

void TIM15_Stop(void)
{
	TIM_Cmd(TIM15, DISABLE);
}

// Configure USART1 COM port.
void USART1_Init_Int(unsigned long baudrate)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    USART_InitTypeDef USART_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;

    // Initialize interrupt tx rx buffers
    tx1headptr = tx1tailptr = rx1headptr = rx1tailptr = 0;

    /* USART1 configured as follow:
    - BaudRate = 115200 baud
    - Word Length = 8 Bits
    - One Stop Bit
    - No parity
    - Hardware flow control disabled (RTS and CTS signals)
    - Receive and transmit enabled
    - PA9 - TX, PA10 - RX     PB6 - TX, PB7 - RX
    */
    USART_InitStructure.USART_BaudRate = baudrate;
    USART_InitStructure.USART_WordLength = USART_WordLength_8b;
    USART_InitStructure.USART_StopBits = USART_StopBits_1;
    USART_InitStructure.USART_Parity = USART_Parity_No;
    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

    /* Enable GPIO clock */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

    /* Enable USART clock */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);

    /* Connect PXx to USARTx_Tx */
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource9, GPIO_AF_1);

    /* Connect PXx to USARTx_Rx */
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource10, GPIO_AF_1);

    /* Configure USART Tx, Rx as alternate function push-pull */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9 | GPIO_Pin_10;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    /* USART configuration */
    USART_Init(USART1, &USART_InitStructure);

    USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);
    USART_ITConfig(USART1, USART_IT_TXE, ENABLE);

	/* Enable the USART3 global Interrupt */
	NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

    /* Enable USART */
    USART_Cmd(USART1, ENABLE);
}

// Initialize the processors peripherals
void STMF042_Init(long baudrate)
{
	// Leds on PA1 and PA0
	LED_Init();   // Leds on PB0 and PB1
	TIM3_Init();  // 1msec pulse interrupt
	TIM15_Init(); // usec delay timer

	// Turn LEDs off
	LED1OFF;
	LED2OFF;

	// Setup USART
	USART1_Init_Int(baudrate);
}

/***********************************************
 *
 *  ISRs
 *
 ***********************************************/
void TIM3_IRQHandler(void)
{
    if (TIM_GetITStatus(TIM3, TIM_IT_CC3) != RESET)  {
		TIM_ClearITPendingBit(TIM3, TIM_IT_CC3);

		if(mseccountdown1) mseccountdown1--;
		if(mseccountdown2) mseccountdown2--;
		if(mseccountdown3) mseccountdown3--;
		TIM_SetCounter(TIM3, 0);
    	}
}

void USART1_IRQHandler(void)
{
	// Check receive interrupt
    if (USART_GetITStatus(USART1, USART_IT_RXNE) != RESET)  {
    	USART_ClearITPendingBit(USART1, USART_IT_RXNE);
    	//LED1LOW;
		rx1buffer[rx1headptr++] = (uint8_t)(USART1->RDR & (uint16_t)0x00FF);
		if(rx1headptr==USART1_RX_BUFSIZE) rx1headptr = 0;
		if(rx1headptr==rx1tailptr) {
			rx1tailptr++;
			if(rx1tailptr==USART1_RX_BUFSIZE) rx1tailptr = 0;
			}
    	}
    // Check transmit interrupt
    if (USART_GetITStatus(USART1, USART_IT_TXE) != RESET)  {
    	USART_ClearITPendingBit(USART1, USART_IT_TXE);
    	//LED2LOW;
		USART1->TDR = tx1buffer[tx1tailptr++];
		if(tx1tailptr==USART1_TX_BUFSIZE) tx1tailptr = 0;
		// Turn off interrupt is not more data to transmit
		if(tx1tailptr==tx1headptr)  USART_ITConfig(USART1, USART_IT_TXE, DISABLE);
    	}
}

/***********************************************
 *
 *  Utility Functions
 *
 ***********************************************/
// Not very accurate - replace with timer based delay
void delay_us(unsigned long ult)
{
	  TIM15_Start();
	  while (TIM_GetCounter(TIM15) < ult)  ;
	  TIM15_Stop();
}

void delay_ms(unsigned int ult)
{
	mseccountdown1 = ult;
	while(mseccountdown1) ;
}

void Toggle_Leds(int grncnt,int redcnt)
{
	if(!mseccountdown2) {
		mseccountdown2 = grncnt;
		if(grnflip) {
			LED1LOW;
			grnflip = 0;
			}
		else {
			LED1HIGH;
			grnflip = 1;
			}
		}
	if(!mseccountdown3) {
		mseccountdown3 = redcnt;
		if(redflip) {
			LED2LOW;
			redflip = 0;
			}
		else {
			LED2HIGH;
			redflip = 1;
			}
		}
}

int GetCharnw1(void)
{
	int intchar;

	if(rx1headptr!=rx1tailptr) {
		intchar = rx1buffer[rx1tailptr++];
		if(rx1tailptr==USART1_RX_BUFSIZE) rx1tailptr = 0;
		return intchar;
		}
	else return -1;
}

unsigned char GetChar1(void)
{
	int intchar;

	while((intchar=GetCharnw1())==-1) ;
	return (unsigned char) intchar;
}

void PutChar1(unsigned char outchar)
{
	if(tx1headptr==tx1tailptr) {
		tx1buffer[tx1headptr++] = outchar;
		if(tx1headptr==USART1_TX_BUFSIZE) tx1headptr = 0;
		USART_ITConfig(USART1, USART_IT_TXE, ENABLE);
		}
	else {
		tx1buffer[tx1headptr++] = outchar;
		if(tx1headptr==USART1_TX_BUFSIZE) tx1headptr = 0;
		if(tx1tailptr==tx1headptr) {
			tx1tailptr++;
			if(tx1tailptr==USART1_TX_BUFSIZE) tx1tailptr = 0;
			}
		}
}

void Puts1(unsigned char *outstrg)
{
	int i;

	for(i=0; i<strlen(outstrg); i++) PutChar1(outstrg[i]);
}

/*************************************************************************
 *
 *   USB character IO functions
 *
 *************************************************************************/
extern uint8_t  APP_Rx_Buffer []; /* Write CDC received data in this buffer.
                                     These data will be sent over USB IN endpoint
                                     in the CDC core functions. */
extern uint32_t APP_Rx_ptr_in;    /* Increment this pointer or roll it back to
                                     start address when writing received data
                                     in the buffer APP_Rx_Buffer. */

/**
  * @brief  VCP_DataTx
  *         CDC received data to be send over USB IN endpoint are managed in
  *         this function.
  * @param  Buf: Buffer of data to be sent
  * @param  Len: Number of data to be sent (in bytes)
  * @retval Result of the operation: USBD_OK if all operations are OK else VCP_FAIL
  */
static uint16_t VCP_DataTx2(uint8_t* Buf, uint32_t Len)
{
	int i;

    for(i=0; i<Len; i++) {
    	APP_Rx_Buffer[APP_Rx_ptr_in++] = Buf[i];
    	//PutChar(Buf[i]);
    	if(APP_Rx_ptr_in == APP_RX_DATA_SIZE)  APP_Rx_ptr_in = 0;
    	}
  return USBD_OK;
}

// USB putchar function
void PutChar(unsigned char outchar)
{
	unsigned char outbuf[2];

	outbuf[0] = outchar;
	VCP_DataTx2(outbuf,1);
}

// USB get char no wait
int GetCharnw(void)
{
	int intchar;

	if(usbrxheadptr!=usbrxtailptr) {
		intchar = usbrxbuffer[usbrxtailptr++];
		if(usbrxtailptr==USB_RX_BUFFERSIZE) usbrxtailptr = 0;
		return intchar;
		}
	else return -1;
}

// USB get char wait
unsigned char GetChar(void)
{
	int intchar;

	while((intchar=GetCharnw())==-1) ;
	return (unsigned int) intchar;
}

void Puts(unsigned char *outstrg)
{
	int i;

	for(i=0; i<strlen(outstrg); i++) PutChar(outstrg[i]);
}

 // Formatted output - memory expensive - will blow memory limit if used
int printf(const char *fmt, ...)
{
	va_list args;
	int i;
	char printbuffer[150];

	va_start (args, fmt);
	i = vsnprintf (printbuffer, 150, fmt, args);
	va_end (args);

	// Print the string
	VCP_DataTx2(printbuffer,i);

	return i;
}

/*************************************************************************
 *
 *   Low memory usage print int and float
 *    *
 *************************************************************************/
// Convert ascii float to float binary
double atofloat(char *s)
{
	double a = 0.0;
	int e = 0;
	int c;
	while ((c = *s++) != '\0' && isdigit(c)) {
		a = a*10.0 + (c - '0');
	}
	if (c == '.') {
		while ((c = *s++) != '\0' && isdigit(c)) {
			a = a*10.0 + (c - '0');
			e = e-1;
		}
	}
	if (c == 'e' || c == 'E') {
		int sign = 1;
		int i = 0;
		c = *s++;
		if (c == '+')
			c = *s++;
		else if (c == '-') {
			c = *s++;
			sign = -1;
		}
		while (isdigit(c)) {
			i = i*10 + (c - '0');
			c = *s++;
		}
		e += i*sign;
	}
	while (e > 0) {
		a *= 10.0;
		e--;
	}
	while (e < 0) {
		a *= 0.1;
		e++;
	}
	return a;
}

// Reverse string s in place
void reverse(char s[])
{
    int i, j;
    char c;

    for (i = 0, j = strlen(s)-1; i<j; i++, j--) {
        c = s[i];
        s[i] = s[j];
        s[j] = c;
    }
}

// Convert int n to characters in s
void itoa(int n, char s[])
{
    int i, sign;

    if ((sign = n) < 0)  /* record sign */
        n = -n;          /* make n positive */
    i = 0;
    do {       /* generate digits in reverse order */
        s[i++] = n % 10 + '0';   /* get next digit */
    } while ((n /= 10) > 0);     /* delete it */
    if (sign < 0)
        s[i++] = '-';
    s[i] = '\0';
    reverse(s);
}

// Low memory print int function
void PrintInt(int i)
{
	char outstrg[10];

	itoa(i,outstrg);
	Puts(outstrg);
}

// Low memory print float function
void Printftoa(float f)
{
	int int_part,frac_part;

	int_part = (int) f;

	frac_part = (f-int_part)*10000;

	PrintInt(int_part);
	PutChar('.');
	PrintInt(frac_part);
}

int main(void)
{
	char inchar,outstrg[20];
	int intchar,status,intvalue;
	float a;

	// Initialize various peripherals including interrupt based USART1
	STMF042_Init(38400);

	// Start the USB CDC Subsystem
	USB_Init();


	// Main processing loop
    while(1) {

    	if((intchar=GetCharnw())!=-1) inchar = (unsigned char) intchar;
    	else inchar = 0;

    	switch (inchar) {
			case 's':
				Puts("\n\rSTM32F042 CDC Example V1.00\n\r>");
				break;
			case '1':
				intvalue = atoi("124");
				Puts("\n\routstrg=");
				PrintInt(intvalue);
				Puts("\n\r>");
				break;
			case '2':
				a = atofloat("6.34");
				Puts("\n\rf=");
				Printftoa(a);
				Puts("\n\r>");
				break;
			case CR:
				Puts("\n\r>");
				break;
    		}
    	//Puts1("hello world");
    	//delay_ms(300);
		Toggle_Leds(100,200);  // Show processor is alive
    	}
}
